package org.garret.scheduler;

import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.io.*;

public class Job implements Runnable, Serializable {
    public static final int EXIT_SUCCESS = 0;
    public static final int EXIT_FAILURE = 1;

    static class StartTime {
        StartTime next;
        Timestamp when;

        StartTime(Timestamp timestamp, StartTime chain) { 
            when = timestamp;
            next = chain;
        }
    }

    transient Job    next;
    transient Job    nextToBeRestarted;
    
    String           name;
    String           homeDir;
    String           params;
    String           className;

    transient String hostName;        
    transient int    port;

    transient Method method;

    static final int STATE_READY    = 0;
    static final int STATE_RUNNING  = 1;
    static final int STATE_FINISHED = 2;
    static final int STATE_FAILED   = 3;   
    transient int    state;

    transient Timestamp started;
    transient Timestamp terminated;
    transient StartTime nextRun;
    
    transient int       runInterval; // seconds
    transient int       runHourMask; // hours at which job should be started
    transient int       runDayOfWeekMask;  // day of week when job should be started
    
    transient int       proceededHourMask;
    transient Object[]  jobParam;

    transient Scheduler sched;

    transient PrintStream      log;
    transient FileOutputStream trace;

    static final int SCHED_TRIGGER   = 0;
    static final int SCHED_PERMANENT = 1;
    static final int SCHED_CALENDAR  = 2;
    static final int SCHED_INTERVAL  = 3;
    static final int SCHED_DAY_HOUR  = 4;
    transient int    schedulePolicy; 
    
    int    restartCount; 
    int    restartCountLimit;
    int    restartDelay; // seconds 
    
    transient Job[]  onFinish;
    transient Job[]  onFailure;

    Job(Scheduler sched, Job list) {
        this.sched = sched;
        next = list;
        jobParam = new Object[1];
        jobParam[0] = this; 
    }

    public final String getJobName() { 
        return name;
    }

    public final String getHostName() { 
        return hostName;
    }

    public final String getHomeDir() { 
        return homeDir;
    }

    public final int getPort() { 
        return port;
    }

    public final String getJobParameters() { 
        return params;
    }

    public final Scheduler getScheduler() { 
        return sched;
    }
    
    public final int getRestartCount() { 
        return restartCount;
    }

    public final int getRestartCountLimit() {
        return restartCountLimit;
    }

    public final int getRestartDelay() {
        return restartDelay;
    }

    public final PrintStream getLogStream() { 
        return log;
    }

    public void run() { 
        for (restartCount = 0; sched.running && restartCount <= restartCountLimit; restartCount++) { 
            try { 
                if (((Integer)method.invoke(null, jobParam)).intValue() == EXIT_SUCCESS) { 
                    sched.endJob(this, STATE_FINISHED);
                    return;
                }
            } catch (Exception x) { 
                sched.traceException(name, x);
            }            
            try { 
                if (restartDelay != 0) { 
                    synchronized (sched.stopEvent) { 
                        sched.stopEvent.wait(restartDelay*1000);
                    }
                }
            } catch (InterruptedException x) {}
        }
        sched.endJob(this, STATE_FAILED);
    }

    void close() { 
        log.close();
    }
}
