package org.garret.scheduler;

import java.sql.*;
import java.util.ArrayList;
import org.garret.jipc.*;

public class QueuePollThread extends Thread {
    static class QueueDesc { 
        String    name;
        String    onOverflowJob;
        JIPCQueue queue;
        int       pending;
        int       threshold;

        QueueDesc(String name, JIPCQueue queue, int threshold, String onOverflowJob) {
            this.name = name;
            this.queue = queue;
            this.threshold = threshold;
            this.onOverflowJob = onOverflowJob;
        }
    }

    QueuePollThread(Scheduler sched) { 
        this.sched = sched;
        try { 
            Statement stmt = sched.connection.createStatement();
            stmt.executeUpdate("delete from QueueState");
            ResultSet cursor = stmt.executeQuery("select * from QueueDesc");
            session = org.garret.jipc.client.JIPCClientFactory.getInstance().create(sched.ipcServerName, sched.ipcServerPort);
            pstmt = sched.connection.prepareStatement("insert into QueueState (qname,pending) values (?,0)");
            ArrayList queueList = new ArrayList();
            while (cursor.next()) {             
                String qName = cursor.getString("qname");
                JIPCQueue queue = null;
                try { 
                    queue = sched.createQueues ? session.createQueue(qName) : session.openQueue(qName);
                } catch (JIPCException x) {}
                queueList.add(new QueueDesc(qName, queue, cursor.getInt("maxelems"), cursor.getString("ovfhnd")));
                pstmt.setString(1, qName);
                pstmt.executeUpdate();
            }
            queues = (QueueDesc[])queueList.toArray(new QueueDesc[queueList.size()]);
            cursor.close();
            pstmt.close();
            stmt.close();
            pstmt = sched.connection.prepareStatement("update QueueState set pending=? where qname=?");
        } catch (Exception x) {
            sched.traceException("POLL-QUEUE", x);
        }
    }

    public void run() { 
        try { 
            while (sched.running) { 
                synchronized (sched.stopEvent) { 
                    sched.stopEvent.wait(sched.queuePollInterval*1000);
                    if (!sched.running) { 
                        break;
                    }
                }

                for (int i = queues.length; --i >= 0;) { 
                    QueueDesc qd = queues[i];
                    if (qd.queue == null) { 
                        try { 
                            qd.queue = session.openQueue(qd.name);
                        } catch (JIPCException x) {}
                    }
                    if (qd.queue != null) { 
                        int pending = qd.queue.size();
                        if (pending != qd.pending) { 
                            pstmt.setInt(1, pending);
                            pstmt.setString(2, qd.name);
                            pstmt.executeUpdate();
                            qd.pending = pending;
                            if (pending > qd.threshold) {
                                sched.startJob(qd.onOverflowJob);
                            }
                        }
                    }
                }
            }
        } catch (Exception x) { 
            sched.traceException("POLL-QUEUE", x);
        }
        try { 
            session.close();
        } catch (Exception x) {}
    }

    QueueDesc[]       queues;
    PreparedStatement pstmt;
    Scheduler         sched;
    JIPCSession       session;
}        
