package org.garret.scheduler; 

import java.io.*;
import java.net.*;
import java.lang.reflect.Method;
import java.sql.Timestamp;

public class RemoteProxy { 
    static class RemoteExecutionThread implements Runnable { 
        Socket socket;

        RemoteExecutionThread(Socket s) { 
            socket = s;
        }

        public void run() {
            try { 
                ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
                OutputStream out = socket.getOutputStream();
                
                Job job = (Job)in.readObject();
                int rc;
                
                job.trace = new FileOutputStream(traceDirectory + job.name + ".out", traceAppend);
                job.log = new PrintStream(job.trace, true);
                job.started = new Timestamp(System.currentTimeMillis());
                job.terminated = null;
                job.log.println("Job '" + job.name + " " + job.params + "' started at " + job.started);
                try { 
                    Class cls = Class.forName(job.className);
                    Method method = cls.getMethod(entryPointMethodName, entryPointMethodSignature);
                    Object[] params = new Object[1];
                    params[0] = job;
                    rc = ((Integer)method.invoke(null, params)).intValue();
                    job.terminated = new Timestamp(System.currentTimeMillis());
                    if (rc == Job.EXIT_SUCCESS) { 
                        job.log.println("Job '" + job.name + " " + job.params + "' finished at " + job.terminated);
                    } else {
                        job.log.println("Job '" + job.name + " " + job.params + "' failed at " + job.terminated);
                    }
                } catch (Exception x) { 
                    x.printStackTrace(log);
                    job.log.println("Job '" + job.name + " " + job.params + "' failed with exception at " + job.terminated);
                    x.printStackTrace(System.err);
                    if (log != null) { 
                        x.printStackTrace(log);
                    }        
                    rc = Job.EXIT_FAILURE;
                }
                out.write(rc);
                socket.close();
            } catch (Exception x) { 
                x.printStackTrace(System.err);
                if (log != null) { 
                    x.printStackTrace(log);
                }        
            }
        }    
    }

    public static void main(String args[]) throws Exception {
        if (args.length != 0) { 
            System.err.println("Usage: java scheduler.RemoteProxy {property}*");
            System.err.println("Properties:");
            System.err.println("\tport:\tport at which proxy will accept connections [6001]");
            System.err.println("\ttrace.dir:\tdirectory where trace files for remote tasks will be created [.]");
            System.err.println("\ttrace.append:\twhether to append or rewrite trace file for remote tasks [false]");
            System.err.println("\tentry.point:\tname of the method which will be invoked by scheduler [run]");
            System.err.println("\tlog.path:\tpath to log file [proxy.log]");
            System.err.println("\tlog.append:\twhether to append or rewrite log file [false]");
        } else {
            entryPointMethodName = System.getProperty("entry.point", "run");

            String logFilePath = System.getProperty("log.path", "proxy.log");
            boolean logAppend = Boolean.getBoolean("log.append");
            if (logFilePath.length() != 0) {
                log = new PrintStream(new FileOutputStream(logFilePath, logAppend), true);
            }
            traceDirectory = System.getProperty("trace.dir", ".");
            if (!traceDirectory.endsWith(File.separator)) { 
                traceDirectory += File.separatorChar;
            }
            traceAppend = Boolean.getBoolean("trace.append");        
            int port = Integer.parseInt(System.getProperty("port", "6001"), 10);
            ServerSocket socket = new ServerSocket(port);
            ThreadPool pool = ThreadPool.getInstance();
            
            while (true) { 
                pool.start(new RemoteExecutionThread(socket.accept()));
            }
        }
    }

    static PrintStream   log;
    static String        traceDirectory;
    static boolean       traceAppend;
    static String        entryPointMethodName;
    static final Class[] entryPointMethodSignature = { Job.class };
}            
